/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s): 
 */

package utobcast.draw;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.io.ByteArrayOutputStream;
import java.io.ObjectOutputStream;
import java.util.Map;
import java.util.Random;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.dream.Push;
import org.objectweb.dream.PushException;
import org.objectweb.dream.message.MessageTypeImpl;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.MessageType;
import org.objectweb.dream.message.manager.MessageManager;
import org.objectweb.dream.util.Error;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.util.Fractal;

/**
 * Shared whiteboard, each new instance joins the same group. Each instance
 * chooses a random color, mouse moves are broadcast to all group members, which
 * then apply them to their canvas
 */
public class DrawImpl extends AbstractComponent implements ActionListener, Push
{
  private ByteArrayOutputStream out         = new ByteArrayOutputStream();
  private String                groupname   = "DrawGroupDemo";
  private int                   memberSize  = 1;
  boolean                       first       = true, cummulative = true;
  private JFrame                mainFrame   = null;
  private JPanel                subPanel    = null;
  private DrawPanel             panel       = null;
  private JButton               clearButton, leaveButton;
  private Random                random      = new Random(System
                                                .currentTimeMillis());
  private final Font            defaultFont = new Font("Helvetica", Font.PLAIN,
                                                12);
  private Color                 drawColor   = selectColor(),
      backgroundColor = Color.white;
  boolean                       noChannel   = false;

  Push                          outPushItf;
  MessageManager                messageManagerItf;

  Component                     nodeComponentItf;
  MessageType                   msgType     = new MessageTypeImpl(
                                                DrawCommandChunk.DEFAULT_NAME,
                                                DrawCommandChunk.TYPE);

  // ---------------------------------------------------------------------------
  // Implementation of the LifeCycleController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.LifeCycleController#startFc()
   */
  public void startFc() throws IllegalLifeCycleException
  {
    super.startFc();

    mainFrame = new JFrame();
    mainFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    panel = new DrawPanel();
    panel.setBackground(backgroundColor);
    subPanel = new JPanel();
    mainFrame.getContentPane().add("Center", panel);
    clearButton = new JButton("Clear");
    clearButton.setFont(defaultFont);
    clearButton.addActionListener(this);
    leaveButton = new JButton("Leave & Exit");
    leaveButton.setFont(defaultFont);
    leaveButton.addActionListener(this);
    subPanel.add("South", clearButton);
    subPanel.add("South", leaveButton);
    mainFrame.getContentPane().add("South", subPanel);
    mainFrame.setBackground(backgroundColor);
    clearButton.setForeground(Color.blue);
    leaveButton.setForeground(Color.blue);
    mainFrame.setTitle("Draw example");
    mainFrame.pack();
    mainFrame.setLocation(15, 25);
    mainFrame.setVisible(true);
  }

  // ---------------------------------------------------------------------------
  // Implementation of the Push interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.dream.Push#push(org.objectweb.dream.message.Message,
   *      java.util.Map)
   */
  public void push(Message message, Map context) throws PushException
  {

    DrawCommand comm = ((DrawCommandChunk) message
        .getChunk(DrawCommandChunk.DEFAULT_NAME)).getDrawCommand();

    switch (comm.mode)
    {
      case DrawCommand.DRAW :
        if (panel != null)
          panel.drawPoint(comm);
        break;
      case DrawCommand.CLEAR :
        clearPanel();
        break;
      default :
        System.err
            .println("***** DrawImpl.run(): received invalid draw command "
                + comm.mode);
        break;
    }
    messageManagerItf.deleteMessage(message);
  }

  // ---------------------------------------------------------------------------
  // Utility methods
  // ---------------------------------------------------------------------------

  private Color selectColor()
  {
    int red = (Math.abs(random.nextInt()) % 255);
    int green = (Math.abs(random.nextInt()) % 255);
    int blue = (Math.abs(random.nextInt()) % 255);
    return new Color(red, green, blue);
  }

  void clearPanel()
  {
    if (panel != null)
      panel.clear();
  }

  void sendClearPanelMsg()
  {
    int tmp[] = new int[1];
    tmp[0] = 0;
    DrawCommand comm = new DrawCommand(DrawCommand.CLEAR);
    ObjectOutputStream os;

    Message msg = messageManagerItf.createMessage(msgType);
    DrawCommandChunk chunk = (DrawCommandChunk) msg
        .getChunk(DrawCommandChunk.DEFAULT_NAME);
    chunk.setDrawCommand(comm);
    try
    {
      outPushItf.push(msg, null);
    }
    catch (PushException e)
    {
      Error.error("Unable to send clear panel message ", logger, e);
    }
  }

  /**
   * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
   */
  public void actionPerformed(ActionEvent e)
  {
    String command = e.getActionCommand();
    if (command.equals("Clear"))
    {
      if (noChannel)
      {
        clearPanel();
        return;
      }

      sendClearPanelMsg();

    }
    else if (command.equals("Leave & Exit"))
    {
      if (!noChannel)
      {
        try
        {
          Runnable stopNode = new StopNode();
          new Thread(stopNode).start();
        }
        catch (Exception ex)
        {
          System.err.println(ex);
        }
      }
      mainFrame.setVisible(false);
      mainFrame.dispose();
      //System.exit(0);
    }
    else
      System.out.println("Unknown action");
  }

  // ---------------------------------------------------------------------------
  // Inner class.
  // ---------------------------------------------------------------------------

  private class DrawPanel extends JPanel implements MouseMotionListener
  {
    Dimension preferredSize = new Dimension(235, 170);
    Image     img           = null;                   // for drawing pixels
    Dimension d, imgsize;
    Graphics  gr            = null;

    public DrawPanel()
    {
      createOffscreenImage();
      addMouseMotionListener(this);
      addComponentListener(new ComponentAdapter()
      {
        public void componentResized(ComponentEvent e)
        {
          if (getWidth() <= 0 || getHeight() <= 0)
            return;
          createOffscreenImage();
        }
      });
    }

    void createOffscreenImage()
    {
      d = getSize();
      if (img == null || imgsize == null || imgsize.width != d.width
          || imgsize.height != d.height)
      {
        img = createImage(d.width, d.height);
        if (img != null)
          gr = img.getGraphics();
        imgsize = d;
      }
    }

    /**
     * Adds pixel to queue and calls repaint() whenever we have MAX_ITEMS pixels
     * in the queue or when MAX_TIME msecs have elapsed (whichever comes first).
     * The advantage compared to just calling repaint() after adding a pixel to
     * the queue is that repaint() can most often draw multiple points at the
     * same time.
     */
    public void drawPoint(DrawCommand c)
    {
      if (c == null || gr == null)
        return;
      gr.setColor(new Color(c.r, c.g, c.b));
      gr.fillOval(c.x, c.y, 10, 10);
      repaint();
    }

    public void clear()
    {
      if (gr == null)
        return;
      gr.clearRect(0, 0, getSize().width, getSize().height);
      repaint();
    }

    public Dimension getPreferredSize()
    {
      return preferredSize;
    }

    public void paintComponent(Graphics g)
    {
      super.paintComponent(g);
      if (img != null)
      {
        g.drawImage(img, 0, 0, null);
      }
    }

    // ---------------------------------------------------------------------------
    // Implementation of the MouseMotionListener interface
    // ---------------------------------------------------------------------------

    public void mouseMoved(MouseEvent e)
    {
    }

    public void mouseDragged(MouseEvent e)
    {
      ObjectOutputStream os;
      int x = e.getX(), y = e.getY();
      DrawCommand comm = new DrawCommand(DrawCommand.DRAW, x, y, drawColor
          .getRed(), drawColor.getGreen(), drawColor.getBlue());

      if (noChannel)
      {
        drawPoint(comm);
        return;
      }

      Message msg = messageManagerItf.createMessage(msgType);
      DrawCommandChunk chunk = (DrawCommandChunk) msg
          .getChunk(DrawCommandChunk.DEFAULT_NAME);
      chunk.setDrawCommand(comm);
      try
      {
        outPushItf.push(msg, null);
      }
      catch (PushException e1)
      {

      }
    }

  }

  // ---------------------------------------------------------------------------
  // Inner class.
  // ---------------------------------------------------------------------------

  private class StopNode implements Runnable
  {

    /**
     * @see java.lang.Runnable#run()
     */
    public void run()
    {
      try
      {
        Fractal.getLifeCycleController(nodeComponentItf).stopFc();
      }
      catch (Exception e)
      {
        e.printStackTrace();
      }
    }

  }

  // ---------------------------------------------------------------------------
  // Implementation of the LifeCycleController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{Push.OUT_PUSH_ITF_NAME, MessageManager.ITF_NAME,
        "node-c"};
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(java.lang.String,
   *      java.lang.Object)
   */
  public void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.bindFc(clientItfName, serverItf);
    if (clientItfName.equals(Push.OUT_PUSH_ITF_NAME))
    {
      outPushItf = (Push) serverItf;
    }
    else if (clientItfName.equals(MessageManager.ITF_NAME))
    {
      messageManagerItf = (MessageManager) serverItf;
    }
    else if (clientItfName.equals("node-c"))
    {
      nodeComponentItf = (Component) serverItf;
    }
  }

}

