/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact : dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): Vivien Quema
 */

package org.objectweb.dream.channel;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.util.Collections;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.dream.control.activity.Util;
import org.objectweb.dream.control.activity.task.AbstractTask;
import org.objectweb.dream.control.activity.task.Task;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * Abstract Accept socket component. This component creates new incoming socket
 * using the {@link #acceptSocket() }method, and pass it through its
 * {@link OpenedSocket }client interface.
 */
public abstract class AbstractAcceptSocketImpl extends AbstractComponent
{

  protected Task         acceptTask = new AcceptTask();

  // ---------------------------------------------------------------------------
  // Client interfaces
  // ---------------------------------------------------------------------------

  protected OpenedSocket openedSocketItf;

  // ---------------------------------------------------------------------------
  // Abstract method
  // ---------------------------------------------------------------------------

  /**
   * Abstract method that return a newly accepted socket.
   */
  protected abstract SocketState acceptSocket() throws IOException;

  // ---------------------------------------------------------------------------
  // Inner task class implementation
  // ---------------------------------------------------------------------------

  protected class AcceptTask extends AbstractTask
  {

    /** Constructor */
    public AcceptTask()
    {
      super("ChannelInTask");
    }

    /**
     * @see Task#execute(Object)
     */
    public Object execute(Object hints) throws InterruptedException
    {
      SocketState socketState;
      ObjectInputStream ois = null;
      try
      {
        logger.log(BasicLevel.DEBUG, "accept socket.");
        socketState = acceptSocket();
        logger.log(BasicLevel.DEBUG, "connected");
      }
      catch (IOException e)
      {
        logger.log(BasicLevel.ERROR,
            "An error occrued while accepting incoming connections", e);
        return EXECUTE_AGAIN;
      }
      try
      {
        openedSocketItf.openedSocket(socketState);
      }
      catch (IOException e)
      {
        logger.log(BasicLevel.WARN, "Input/Output socket error", e);
        socketState.close();
      }

      return EXECUTE_AGAIN;
    }
  }

  // -------------------------------------------------------------------------
  // Overriden methods
  // -------------------------------------------------------------------------

  /**
   * @see org.objectweb.dream.AbstractComponent#beforeFirstStart(org.objectweb.fractal.api.Component)
   */
  protected void beforeFirstStart(Component componentItf)
      throws IllegalLifeCycleException
  {
    try
    {
      Util.addTask(componentItf, acceptTask, Collections.EMPTY_MAP);
    }
    catch (Exception e)
    {
      throw new IllegalLifeCycleException("Can't add task");
    }
  }

  // ---------------------------------------------------------------------------
  // Implementation of BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{OpenedSocket.ITF_NAME};
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(String,
   *      Object)
   */
  public void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.bindFc(clientItfName, serverItf);
    if (clientItfName.equals(OpenedSocket.ITF_NAME))
    {
      openedSocketItf = (OpenedSocket) serverItf;
    }
  }
}