/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.channel;

import java.io.IOException;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.dream.message.ExtensibleMessage;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.manager.MessageManager;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;

/**
 * Socket manager that retrieves sockets using an {@link IPSocketManager }and
 * getting IP and port in a {@link IPChannelDestinationChunk }in messages.
 */
public class SocketManagerIPChunkBasedImpl extends AbstractComponent
    implements
      SocketManager,
      SocketManagerIPChunkBasedAttributeController
{

  // ---------------------------------------------------------------------------
  // attribute fields
  // ---------------------------------------------------------------------------

  protected boolean         deleteChunk = true;
  protected String          destinationChunkName;

  // ---------------------------------------------------------------------------
  // Client interfaces
  // ---------------------------------------------------------------------------

  protected MessageManager  messageManagerItf;
  protected IPSocketManager delegateSocketManagerItf;

  // ---------------------------------------------------------------------------
  // Implementation of SocketManager interface
  // ---------------------------------------------------------------------------

  /**
   * @see SocketManager#getSocket(Message)
   */
  public SocketState getSocket(Message message) throws IOException,
      InterruptedException
  {
    IPChannelDestinationChunk chunk = (IPChannelDestinationChunk) message
        .getChunk(destinationChunkName);
    if (chunk == null)
    {
      throw new IOException("Unable to find \"" + destinationChunkName
          + "\" chunk in message");
    }
    SocketState socketState = delegateSocketManagerItf.getSocket(chunk
        .getChannelDestinationAddr(), chunk.getChannelDestinationPort());
    if (deleteChunk && (message instanceof ExtensibleMessage))
    {
      ((ExtensibleMessage) message).removeChunk(destinationChunkName);
      messageManagerItf.deleteChunk(chunk);
    }
    return socketState;
  }

  /**
   * @see SocketManager#releaseSocket(SocketState, boolean)
   */
  public void releaseSocket(SocketState socketState, boolean error)
  {
    delegateSocketManagerItf.releaseSocket(socketState, error);
  }

  // ---------------------------------------------------------------------------
  // Implementation of AttributeController interface
  // ---------------------------------------------------------------------------

  /**
   * @see SocketManagerIPChunkBasedAttributeController#getDeleteChunk()
   */
  public boolean getDeleteChunk()
  {
    return deleteChunk;
  }

  /**
   * @see SocketManagerIPChunkBasedAttributeController#setDeleteChunk(boolean)
   */
  public void setDeleteChunk(boolean deleteChunk)
  {
    this.deleteChunk = deleteChunk;
  }

  /**
   * @see SocketManagerIPChunkBasedAttributeController#getDestinationChunkName()
   */
  public String getDestinationChunkName()
  {
    return destinationChunkName;
  }

  /**
   * @see SocketManagerIPChunkBasedAttributeController#setDestinationChunkName(String)
   */
  public void setDestinationChunkName(String destinationChunkName)
  {
    this.destinationChunkName = destinationChunkName;
  }

  // ---------------------------------------------------------------------------
  // Implementation of BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{MessageManager.ITF_NAME, IPSocketManager.ITF_NAME};
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(String,
   *      Object)
   */
  public synchronized void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.bindFc(clientItfName, serverItf);
    if (clientItfName.equals(MessageManager.ITF_NAME))
    {
      messageManagerItf = (MessageManager) serverItf;
    }
    else if (clientItfName.equals(IPSocketManager.ITF_NAME))
    {
      delegateSocketManagerItf = (IPSocketManager) serverItf;
    }
  }
}