/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s): 
 */

package org.objectweb.dream.protocol.utobcast;

import java.util.Map;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.dream.Push;
import org.objectweb.dream.PushException;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.protocol.utobcast.message.UTOBcastChunk;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * Implementation of the IncomingMessageRouter component. This component routes
 * incoming messages on one of its outputs. These outputs correspond to the
 * different types of messages that can be exchanged (i.e. DAT, ACK, REP, UTO,
 * UPD, BAK).
 */
public class IncomingMessageRouterImpl extends AbstractComponent
    implements
      Push
{

  // ---------------------------------------------------------------------------
  // Fields of this class
  // ---------------------------------------------------------------------------

  /** The outputs. */
  protected Push[] outPushItfs = new Push[6];

  // ---------------------------------------------------------------------------
  // Constructor
  // ---------------------------------------------------------------------------

  /**
   * Constructor.
   */
  public IncomingMessageRouterImpl()
  {
  }

  // ---------------------------------------------------------------------------
  // Implementation of the Push interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.dream.Push#push(org.objectweb.dream.message.Message,
   *      java.util.Map)
   */
  public void push(Message message, Map context) throws PushException
  {
    // TODO Auto-generated method stub
    UTOBcastChunk chunk = (UTOBcastChunk) message
        .getChunk(UTOBcastChunk.DEFAULT_NAME);
    if (chunk == null)
    {
      logger.log(BasicLevel.ERROR,
          "The message does not contain an UTOBcast chunk:" + message);
      throw new PushException("The message does not contain an UTOBcast chunk:"
          + message);
    }
    outPushItfs[chunk.getUTOBcastMessageType()].push(message, null);
  }

  // ---------------------------------------------------------------------------
  // Implementation of the BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(java.lang.String,
   *      java.lang.Object)
   */
  public void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.bindFc(clientItfName, serverItf);
    if (clientItfName.endsWith("DAT"))
    {
      outPushItfs[UTOBcastChunk.DAT] = (Push) serverItf;
    }
    else if (clientItfName.endsWith("ACK"))
    {
      outPushItfs[UTOBcastChunk.ACK] = (Push) serverItf;
    }
    else if (clientItfName.endsWith("REP"))
    {
      outPushItfs[UTOBcastChunk.REP] = (Push) serverItf;
    }
    else if (clientItfName.endsWith("UTO"))
    {
      outPushItfs[UTOBcastChunk.UTO] = (Push) serverItf;
    }
    else if (clientItfName.endsWith("UPD"))
    {
      outPushItfs[UTOBcastChunk.UPD] = (Push) serverItf;
    }
    else if (clientItfName.endsWith("BAK"))
    {
      outPushItfs[UTOBcastChunk.BAK] = (Push) serverItf;
    }
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(java.lang.String,
   *      java.lang.Object)
   */
  public void unbindFc(String clientItfName) throws NoSuchInterfaceException,
      IllegalBindingException, IllegalLifeCycleException
  {
    super.unbindFc(clientItfName);
    if (clientItfName.endsWith("DAT"))
    {
      outPushItfs[UTOBcastChunk.DAT] = null;
    }
    else if (clientItfName.endsWith("ACK"))
    {
      outPushItfs[UTOBcastChunk.ACK] = null;
    }
    else if (clientItfName.endsWith("REP"))
    {
      outPushItfs[UTOBcastChunk.REP] = null;
    }
    else if (clientItfName.endsWith("UTO"))
    {
      outPushItfs[UTOBcastChunk.UTO] = null;
    }
    else if (clientItfName.endsWith("UPD"))
    {
      outPushItfs[UTOBcastChunk.UPD] = null;
    }
    else if (clientItfName.endsWith("BAK"))
    {
      outPushItfs[UTOBcastChunk.BAK] = null;
    }
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    String[] tab = new String[outPushItfs.length];
    tab[0] = Push.OUT_PUSH_ITF_NAME + "-DAT";
    tab[1] = Push.OUT_PUSH_ITF_NAME + "-ACK";
    tab[2] = Push.OUT_PUSH_ITF_NAME + "-REP";
    tab[3] = Push.OUT_PUSH_ITF_NAME + "-UTO";
    tab[4] = Push.OUT_PUSH_ITF_NAME + "-UPD";
    tab[5] = Push.OUT_PUSH_ITF_NAME + "-BAK";
    return tab;
  }

}