/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s): 
 */

package org.objectweb.dream.queue;

import org.objectweb.dream.AbstractComponent;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.manager.MessageManager;
import org.objectweb.dream.util.Error;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;

/**
 * Abstract implementation of a buffer. It implements
 * {@link PushQueueAttributeController},{@link Buffer},
 * {@link BufferAddFirstLast}, and {@link BufferRemoveFirstLast}interfaces.
 * <p>
 * Implementations of <code>add</code>, and <code>remove</code> methods
 * (and their variants: <code>addFirst</code>,<code>addLast</code>,
 * <code>removeFirst</code>,<code>removeLast</code>,
 * <code>getFirst</code>, and <code>getLast</code>) are implemented as
 * follows
 * <ul>
 * <li>The <code>add</code> method checks for available space and then calls
 * the <code>doAdd</code> method to add the message to the buffer.</li>
 * <li>The <code>remove</code> method checks for available messages and then
 * calls the <code>doRemove</code> method to remove a message from the buffer.
 * </li>
 * </ul>
 * <p>
 * Buffer developers can inherit this class. They must implement several
 * methods:
 * <ul>
 * <li><code>canAdd(message)</code> returns a boolean indicating whether the
 * given message can be added into the buffer.</li>
 * <li><code>doAdd(message)</code> adds the given message into the buffer.
 * </li>
 * <li><code>hasAvailableMessage()</code> returns a boolean indicating
 * whether there is an available message.</li>
 * <li><code>doRemove()</code> removes a message from the buffer.</li>
 * <li>etc.</li>
 * </ul>
 * <p>
 * Moreover developers must use the
 * <code>incrementAvailableMessagesIndicator</code> and
 * <code>incrementStoredMessagesIndicator</code> methods to increment
 * indicators on stored messages, available messages and available space.
 * <p>
 * <i>Note: </i> the <code>add</code>,<code>remove</code>,
 * <code>addFirst</code>,<code>addLast</code>,<code>removeFirst</code>,
 * <code>removeLast</code>,<code>getFirst</code>, and
 * <code>getLast</code> methods should not be overriden.
 */
public abstract class AbstractBufferImpl extends AbstractComponent
    implements
      Buffer,
      BufferAddFirstLast,
      BufferRemoveFirstLast,
      QueueAttributeController
{

  /** The <code>MessageManager</code> client interface. */
  protected MessageManager messageManagerItf;

  /** The maximal capacity of the buffer. */
  protected int            maxCapacity                = 0;

  /**
   * An indicator on available messages into the buffer. This indicator should
   * only be updated using the <code>incrementAvailableMessagesIndicator</code>
   * method.
   */
  protected int            availableMessagesIndicator = 0;

  /**
   * An indicator on stored messages into the buffer. This indicator should only
   * be updated using the <code>incrementStoredMessagesIndicator</code>
   * method.
   */
  protected int            storedMessagesIndicator    = 0;

  /** A lock object. */
  protected final Object   lock                       = new Object();

  // ---------------------------------------------------------------------------
  // Implementation of the Buffer interface.
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.dream.queue.Buffer#availableMessagesIndicator()
   */
  public int availableMessagesIndicator()
  {
    return availableMessagesIndicator;
  }

  /**
   * @see org.objectweb.dream.queue.Buffer#storedMessagesIndicator()
   */
  public int storedMessagesIndicator()
  {
    return storedMessagesIndicator;
  }

  /**
   * @see org.objectweb.dream.queue.Buffer#availableSpaceIndicator()
   */
  public int availableSpaceIndicator()
  {
    if (maxCapacity <= 0)
    {
      return Integer.MAX_VALUE;
    }
    else
    {
      return maxCapacity - storedMessagesIndicator;
    }
  }

  /**
   * This method should not be overriden.
   * 
   * @see org.objectweb.dream.queue.Buffer#add(org.objectweb.dream.message.Message)
   */
  public void add(Message message) throws InterruptedException
  {
    synchronized (lock)
    {
      while (!canAdd(message))
      {
        lock.wait();
      }
      doAdd(message);
    }
  }

  /**
   * This method should not be overriden.
   * 
   * @see org.objectweb.dream.queue.Buffer#remove()
   */
  public Message remove() throws InterruptedException
  {
    synchronized (lock)
    {
      while (!hasAvailableMessage())
      {
        lock.wait();
      }
      return doRemove();
    }
  }

  /**
   * This method should not be overriden.
   * 
   * @see org.objectweb.dream.queue.Buffer#get()
   */
  public Message get() throws InterruptedException
  {
    synchronized (lock)
    {
      while (!hasAvailableMessage())
      {
        lock.wait();
      }
      return doGet();
    }
  }

  /**
   * @see org.objectweb.dream.queue.BufferAddFirstLast#addFirst(org.objectweb.dream.message.Message)
   */
  public void addFirst(Message message) throws InterruptedException
  {
    synchronized (lock)
    {
      while (!canAdd(message))
      {
        lock.wait();
      }
      doAddFirst(message);
    }
  }

  /**
   * @see org.objectweb.dream.queue.BufferAddFirstLast#addLast(org.objectweb.dream.message.Message)
   */
  public void addLast(Message message) throws InterruptedException
  {
    synchronized (lock)
    {
      while (!canAdd(message))
      {
        lock.wait();
      }
      doAddLast(message);
    }
  }

  /**
   * @see org.objectweb.dream.queue.BufferRemoveFirstLast#getFirst()
   */
  public Message getFirst() throws InterruptedException
  {
    synchronized (lock)
    {
      while (!hasAvailableMessage())
      {
        lock.wait();
      }
      return doGetFirst();
    }
  }

  /**
   * @see org.objectweb.dream.queue.BufferRemoveFirstLast#getLast()
   */
  public Message getLast() throws InterruptedException
  {
    synchronized (lock)
    {
      while (!hasAvailableMessage())
      {
        lock.wait();
      }
      return doGetLast();
    }
  }

  /**
   * @see org.objectweb.dream.queue.BufferRemoveFirstLast#removeFirst()
   */
  public Message removeFirst() throws InterruptedException
  {
    synchronized (lock)
    {
      while (!hasAvailableMessage())
      {
        lock.wait();
      }
      return doRemoveFirst();
    }
  }

  /**
   * @see org.objectweb.dream.queue.BufferRemoveFirstLast#removeLast()
   */
  public Message removeLast() throws InterruptedException
  {
    synchronized (lock)
    {
      while (!hasAvailableMessage())
      {
        lock.wait();
      }
      return doRemoveLast();
    }
  }

  // ---------------------------------------------------------------------------
  // Utility methods.
  // ---------------------------------------------------------------------------

  /**
   * Increments the indicator on available messages.
   * 
   * @param delta the delta that must be added to previous indicator.
   */
  protected void incrementAvailableMessagesIndicator(int delta)
  {
    synchronized (lock)
    {
      availableMessagesIndicator += delta;
      lock.notifyAll();
    }
  }

  /**
   * Increments the indicator on stored messages.
   * 
   * @param delta the delta that must be added to previous indicator.
   */
  protected void incrementStoredMessagesIndicator(int delta)
  {
    synchronized (lock)
    {
      storedMessagesIndicator += delta;
      lock.notifyAll();
    }
  }

  // ---------------------------------------------------------------------------
  // Methods that must be implemented by concrete super classes.
  // ---------------------------------------------------------------------------

  /**
   * Checks whether the given message can be added into the buffer.
   * 
   * @param message the message to be tested.
   * @return <code>true</code> if the message can be added.
   */
  protected abstract boolean canAdd(Message message);

  /**
   * Adds a message to the buffer. This method should not check if there is
   * enough available space provided it has already been done in the
   * {@link #add}method.
   * 
   * @param message the message to be added.
   */
  protected abstract void doAdd(Message message);

  /**
   * Checks whether there is an available message.
   * 
   * @return <code>true</code> if there is an available message.
   */
  protected abstract boolean hasAvailableMessage();

  /**
   * Removes a message from the buffer. This method should not check if there is
   * a message available provided it has already been done in the
   * {@link #remove}method.
   * 
   * @return a message.
   */
  protected abstract Message doRemove();

  /**
   * Gets a message from the buffer. This method should not check if there is a
   * message available provided it has already been done in the {@link #remove}
   * method.
   * 
   * @return a message.
   */
  protected abstract Message doGet();

  // ---------------------------------------------------------------------------
  // Methods that may be overriden by concrete super classes.
  // ---------------------------------------------------------------------------

  /**
   * Adds a message to the beginning of the buffer. This method should not check
   * if there is enough available space provided it has already been done in the
   * {@link #add}method.
   * 
   * @param message the message to be added.
   */
  protected void doAddFirst(Message message)
  {
    Error.bug(logger, new UnsupportedOperationException(
        "This method is not implemented"));
  }

  /**
   * Adds a message to end of the buffer. This method should not check if there
   * is enough available space provided it has already been done in the
   * {@link #add}method.
   * 
   * @param message the message to be added.
   */
  protected void doAddLast(Message message)
  {
    Error.bug(logger, new UnsupportedOperationException(
        "This method is not implemented"));
  }

  /**
   * Gets the first message from the buffer. This method should not check if
   * there is a message available provided it has already been done in the
   * {@link #remove}method.
   * 
   * @return a message.
   */
  protected Message doGetFirst()
  {
    Error.bug(logger, new UnsupportedOperationException(
        "This method is not implemented"));
    return null;
  }

  /**
   * Gets the last message from the buffer. This method should not check if
   * there is a message available provided it has already been done in the
   * {@link #remove}method.
   * 
   * @return a message.
   */
  protected Message doGetLast()
  {
    Error.bug(logger, new UnsupportedOperationException(
        "This method is not implemented"));
    return null;
  }

  /**
   * Removes the first message from the buffer. This method should not check if
   * there is a message available provided it has already been done in the
   * {@link #remove}method.
   * 
   * @return a message.
   */
  protected Message doRemoveFirst()
  {
    Error.bug(logger, new UnsupportedOperationException(
        "This method is not implemented"));
    return null;
  }

  /**
   * Removes the last message from the buffer. This method should not check if
   * there is a message available provided it has already been done in the
   * {@link #remove}method.
   * 
   * @return a message.
   */
  protected Message doRemoveLast()
  {
    Error.bug(logger, new UnsupportedOperationException(
        "This method is not implemented"));
    return null;
  }

  // ---------------------------------------------------------------------------
  // Implementation of the QueueAttributeController interface.
  // ---------------------------------------------------------------------------

  /**
   * @see QueueAttributeController#getMaxCapacity()
   */
  public int getMaxCapacity()
  {
    synchronized (lock)
    {
      return maxCapacity;
    }
  }

  /**
   * @see QueueAttributeController#setMaxCapacity(int)
   */
  public void setMaxCapacity(int maxCapacity)
  {
    synchronized (lock)
    {
      int previousCapacity = this.maxCapacity;
      this.maxCapacity = maxCapacity;
      if (maxCapacity > previousCapacity)
      {
        // notify blocking add methods.
        lock.notifyAll();
      }
    }
  }

  /**
   * @see org.objectweb.dream.queue.QueueAttributeController#getCurrentSize()
   */
  public int getCurrentSize()
  {
    return storedMessagesIndicator;
  }

  // ---------------------------------------------------------------------------
  // Implementation of the BindingController interface.
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(String,
   *      Object)
   */
  public synchronized void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.bindFc(clientItfName, serverItf);
    if (clientItfName.equals(MessageManager.ITF_NAME))
    {
      messageManagerItf = (MessageManager) serverItf;
    }
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{MessageManager.ITF_NAME};
  }
}