/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Vivien Quema
 * Contributor(s): 
 */

package org.objectweb.dream.queue;

import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.manager.MessageManager;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;

/**
 * Basic implementation of the <code>Buffer</code> interface using a
 * <code>LinkedList</code>. This buffer does not sort messages. They are
 * added and removed in a FIFO order.
 * <p>
 * <i>Indicators </i> on available space, available messages, and stored
 * messages are expressed as number of messages.
 * 
 * @see Buffer
 * @see AbstractBufferImpl
 */
public class BufferImpl extends AbstractBufferImpl
{

  ListAddFirstLast    listAddFirstLastItf;
  ListRemoveFirstLast listRemoveFirstLastItf;
  List                listItf;

  // ---------------------------------------------------------------------------
  // Implemented methods.
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.dream.queue.AbstractBufferImpl#hasAvailableMessage()
   */
  protected boolean hasAvailableMessage()
  {
    return (availableMessagesIndicator > 0);
  }

  /**
   * @see org.objectweb.dream.queue.AbstractBufferImpl#canAdd(org.objectweb.dream.message.Message)
   */
  protected boolean canAdd(Message message)
  {
    if (maxCapacity > 0 && (maxCapacity - storedMessagesIndicator <= 0))
    {
      return false;
    }
    return true;
  }

  // ---------------------------------------------------------------------------
  // Implemented methods for the Buffer interface.
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.dream.queue.AbstractBufferImpl#doAdd(org.objectweb.dream.message.Message)
   */
  protected void doAdd(Message message)
  {
    listItf.add(message);
    incrementAvailableMessagesIndicator(1);
    incrementStoredMessagesIndicator(1);
  }

  /**
   * @see org.objectweb.dream.queue.AbstractBufferImpl#doRemove()
   */
  protected Message doRemove()
  {
    incrementAvailableMessagesIndicator(-1);
    incrementStoredMessagesIndicator(-1);
    return (Message) listRemoveFirstLastItf.removeFirst();
  }

  /**
   * @see org.objectweb.dream.queue.AbstractBufferImpl#doGet()
   */
  protected Message doGet()
  {
    return (Message) listRemoveFirstLastItf.getFirst();
  }

  // ---------------------------------------------------------------------------
  // Implemented methods for the BufferAddFirstLast interface.
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.dream.queue.AbstractBufferImpl#doAddFirst(org.objectweb.dream.message.Message)
   */
  protected void doAddFirst(Message message)
  {
    listAddFirstLastItf.addFirst(message);
    incrementAvailableMessagesIndicator(1);
    incrementStoredMessagesIndicator(1);
  }

  /**
   * @see org.objectweb.dream.queue.AbstractBufferImpl#doAddLast(org.objectweb.dream.message.Message)
   */
  protected void doAddLast(Message message)
  {
    listAddFirstLastItf.addLast(message);
    incrementAvailableMessagesIndicator(1);
    incrementStoredMessagesIndicator(1);
  }

  // ---------------------------------------------------------------------------
  // Implemented methods for the BufferRemoveFirstLast interface.
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.dream.queue.AbstractBufferImpl#doGetFirst()
   */
  protected Message doGetFirst()
  {
    return (Message) listRemoveFirstLastItf.getFirst();
  }

  /**
   * @see org.objectweb.dream.queue.AbstractBufferImpl#doGetLast()
   */
  protected Message doGetLast()
  {
    return (Message) listRemoveFirstLastItf.getLast();
  }

  /**
   * @see org.objectweb.dream.queue.AbstractBufferImpl#doRemoveFirst()
   */
  protected Message doRemoveFirst()
  {
    incrementAvailableMessagesIndicator(-1);
    incrementStoredMessagesIndicator(-1);
    return (Message) listRemoveFirstLastItf.removeFirst();
  }

  /**
   * @see org.objectweb.dream.queue.AbstractBufferImpl#doRemoveLast()
   */
  protected Message doRemoveLast()
  {
    incrementAvailableMessagesIndicator(-1);
    incrementStoredMessagesIndicator(-1);
    return (Message) listRemoveFirstLastItf.removeLast();
  }

  // ---------------------------------------------------------------------------
  // Implementation of the BindingController interface.
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(String,
   *      Object)
   */
  public synchronized void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.bindFc(clientItfName, serverItf);
    if (clientItfName.equals(List.ITF_NAME))
    {
      listItf = (List) serverItf;
    }
    else if (clientItfName.equals(ListAddFirstLast.ITF_NAME))
    {
      listAddFirstLastItf = (ListAddFirstLast) serverItf;
    }
    else if (clientItfName.equals(ListRemoveFirstLast.ITF_NAME))
    {
      listRemoveFirstLastItf = (ListRemoveFirstLast) serverItf;
    }
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{MessageManager.ITF_NAME, ListAddFirstLast.ITF_NAME,
        ListRemoveFirstLast.ITF_NAME, List.ITF_NAME};
  }

}